import logging
from typing import List, Tuple, Union, Dict, Any


from langchain_core.messages import HumanMessage, SystemMessage, BaseMessage

from gtagger.model import Document
from gtagger.config import get_config

config = get_config()

log = logging.getLogger(__name__)


class Ai:
    """
    Classe per l'interazione con un modello LLM via LangChain,
    con supporto per output strutturato e raccolta delle statistiche d'uso.
    La logica di inizializzazione del modello viene gestita esternamente.
    """

    def __init__(self, model):
        """
        Inizializza l'oggetto Ai con un modello LLM già costruito.
        """
        self.model = model

    # def __init__(self):
    #     """
    #     Inizializza il modello LLM usando il provider appropriato
    #     in base alla configurazione.
    #     """
    #     # google_genai google_vertexai
    #     provider = "google_genai" if config.env == "development" else "google_vertexai"
    #     try:
    #         self.model = init_chat_model(config.model, model_provider=provider)
    #     except:
    #         log.warning("Vertex not started")


    def invoke(self, messages: List[BaseMessage]) -> Any:
        """
        Invia una lista di messaggi al modello e restituisce la risposta.

        Args:
            messages (List[BaseMessage]): Lista di messaggi da inviare al modello.

        Returns:
            Any: La risposta generata dal modello.
        """        
        log.debug("message: %s", messages)
        
        try:
            response = self.model.invoke(messages)
        except AttributeError:
            log.error("no GOOGLE API access")
            raise SystemExit
        
        log.debug("response: %s", response)
        return response


    def structured_invoke(self, template: Document, messages: List[BaseMessage]) -> Tuple[Document, str]:
        """
        Invia i messaggi al modello usando l'output strutturato e restituisce
        sia i dati che le statistiche d'uso.
        """
        log.debug("message: %s", messages)
        try:
            llm = self.model.with_structured_output(template, include_raw=True)
        except AttributeError:
            log.error("no GOOGLE API access")
            raise SystemExit

        response = llm.invoke(messages)
        data = response.get("parsed")
        aim=response.get('raw')
        usage = aim.usage_metadata

        log.debug("response: %s", response)

        return data, usage
